const chalk = require('chalk');
const Promise = require('bluebird');
const fs = require('fs');
const path = require('path');
const dateTime = require('node-datetime');
const request = Promise.promisify(require('request'));
const zipdir = Promise.promisify(require('zip-dir'));
const { notify } = require('./util');

const dt = dateTime.create();
const formattedDate = dt.format('Y-m-d-H-M-S');

const environment = process.env.NODE_ENV.toUpperCase();
const omittedDefault = /node_modules$|^.*\.(scss|map|less)$|.git$|__tests__$|build$|src$|package*$|yarn.lock$|circle[.]*|bash[.]*|gulp[.]*|docker[.]*|deploy[.]*|deployment[.]*|pagebuilder[.]*|mongo[.]*|proxy[.]*/;

/**
* @desc Bundles and prepares the PageBuilder assets so it can be posted to the API endpoint. Creates a .zip file.
* @param {string} omittedContent - A regex expression that correlates to the content that should be omitted from the bundle zip.
**/
bundle = (omittedContent = omittedDefault) => {
  console.log(`${chalk.green('Bundling the project to a zip file...')} `);

  const rootDirectory = path.resolve(__dirname, '..', '..', '..');
  const bundleDirectory = path.join(rootDirectory, '/deployment/');
  const bundleFile = `${bundleDirectory}${formattedDate}.zip`;

  // Creates the bundle directory if it doesn't exist.
  if (!fs.existsSync(bundleDirectory)) {
    fs.mkdirSync(bundleDirectory);
  }

  const options = {
    filter: (path, stat) => !(omittedContent.test(path)),
    saveTo: bundleFile,
    each: path => console.log(`${chalk.yellow(`${path} added to zip`)}`)
  };

  const cb = (err, buffer) => {
    if (err) {
      console.error(err, 'Aborting...');
    } else {
      console.log(`${chalk.green('Bundle complete! Bundle zip drive saved to')} ${bundleFile}`);
      deploy(bundleFile, formattedDate);
    }
  };

  return setTimeout(() => {
    zipdir(rootDirectory, options, cb);
  }, 2000);
};

/**
* @desc Posts the .zip file to the deployer API based on the environment.
* @param {string} path - A string describing the location of the generted .zip file.
* @param {string} name - A string containing the name of the generated .zip file.
**/
deploy = (path, name) => {
  console.log(`${chalk.green('Deploying to PageBuilder with')} ${path}`);

  // Gets environment variables from CircleCI.
  const endpoint = process.env[`${environment}_DEPLOYER_ENDPOINT`];
  const username = process.env[`${environment}_DEPLOYER_USERNAME`];
  const password = process.env[`${environment}_DEPLOYER_PASSWORD`];

  console.log(`${chalk.green(`Deploying to ${environment} environment...`)}`);

  const options = {
    url: `https://${username}:${password}@${endpoint}/deployments/pagebuilder/bundles`,
    formData: {
      name,
      bundle: fs.createReadStream(path)
    }
  };

  request.post(options, (error, response, body) => {
    if (error || response.statusCode !== 200) {
      console.log(`${chalk.red('Deployment error has occurred!')}`);
      console.log(`${chalk.red(error)}`);

      notify(`Deployment of PageBuilder bundle \`${name}\` to \`${process.env.NODE_ENV}\` has *failed*. Please see the CircleCI logs for more details. :angryparrot:`);
    } else {
      console.log(`${chalk.green(`Response: ${response}, ${body}`)}`);
      console.log(`${chalk.green(`Deployment of ${name} to ${process.env.NODE_ENV} complete. The bundle can now be promoted in the PageBuilder deployer window.`)}`);

      notify(`PageBuilder bundle \`${name}\` has been deployed to the \`${process.env.NODE_ENV}\` environment and is ready for promotion. :partyparrot:`);
    }
  });
};

module.exports = bundle;
